@@ -10,7 +10,6 @@ import ssl
 import glob
 import datetime
 import hashlib
-import sqlite3
 from ssl import CertificateError
 import ansicat
 import offutils
@@ -158,7 +157,7 @@ def get_cache_path(url,add_index=True):
     else:
         local = False
         # Convert unicode hostname to punycode using idna RFC3490
-        host = parsed.hostname #.encode("idna").decode()
+        host = parsed.netloc #.encode("idna").decode()
         port = parsed.port or standard_ports.get(scheme, 0)
         # special gopher selector case
         if scheme == "gopher":
@@ -500,83 +499,94 @@ def _validate_cert(address, host, cert,accept_bad_ssl=False,automatic_choice=Non
     sha.update(cert)
     fingerprint = sha.hexdigest()
 
-    db_path = os.path.join(xdg("config"), "tofu.db")
-    db_conn = sqlite3.connect(db_path)
-    db_cur = db_conn.cursor()
-
-    db_cur.execute("""CREATE TABLE IF NOT EXISTS cert_cache
-        (hostname text, address text, fingerprint text,
-        first_seen date, last_seen date, count integer)""")
-    # Have we been here before?
-    db_cur.execute("""SELECT fingerprint, first_seen, last_seen, count
-        FROM cert_cache
-        WHERE hostname=? AND address=?""", (host, address))
-    cached_certs = db_cur.fetchall()
-
-    # If so, check for a match
-    if cached_certs:
+    # The directory of this host and IP-address, e.g.
+    # ~/.local/share/offpunk/certs/srht.site/46.23.81.157/
+    certdir = os.path.join(xdg("data"), "certs")
+    hostdir = os.path.join(certdir, host)
+    sitedir = os.path.join(hostdir, address)
+    #1. We check through cached certificates do extract the 
+    #most_frequent_cert  and to see if one is matching the current one.
+    #2. If we have no match but one valid most_frequent_cert, we do the 
+    #"throws warning" code.
+    #3. If no certificate directory or no valid cached certificates, we do 
+    #the "First-Use" routine.
+    most_frequent_cert = None
+    matching_fingerprint = False
+    # 1. Have we been here before? (the directory exists)
+    if os.path.isdir(sitedir):
         max_count = 0
-        most_frequent_cert = None
-        for cached_fingerprint, first, last, count in cached_certs:
-            if count > max_count:
-                max_count = count
-                most_frequent_cert = cached_fingerprint
-            if fingerprint == cached_fingerprint:
-                # Matched!
-                db_cur.execute("""UPDATE cert_cache
-                    SET last_seen=?, count=?
-                    WHERE hostname=? AND address=? AND fingerprint=?""",
-                    (now, count+1, host, address, fingerprint))
-                db_conn.commit()
-                break
+        files = os.listdir(sitedir)
+        count = 0
+        certcache = os.path.join(xdg("config"), "cert_cache")
+
+        for cached_fingerprint in files:
+            filepath = os.path.join(sitedir, cached_fingerprint)
+            certpath = os.path.join(certcache,cached_fingerprint+".crt")
+            with open(filepath, 'r') as f:
+                count = int(f.read())
+            if os.path.exists(certpath):
+                if count > max_count:
+                    max_count = count
+                    most_frequent_cert = cached_fingerprint
+                if fingerprint == cached_fingerprint:
+                    # Matched!
+                    # Increase the counter for this certificate (this also updates
+                    # the modification time of the file)
+                    with open(filepath, 'w') as f:
+                        f.write(str(count+1))
+                    matching_fingerprint = True
+                    break
+    #2. Do we have some certificates but none of them is matching the current one?
+    if most_frequent_cert and not matching_fingerprint:
+        with open(os.path.join(certcache, most_frequent_cert+".crt"), "rb") as fp:
+            previous_cert = fp.read()
+        if _HAS_CRYPTOGRAPHY:
+            # Load the most frequently seen certificate to see if it has
+            # expired
+            previous_cert = x509.load_der_x509_certificate(previous_cert, _BACKEND)
+            previous_ttl = previous_cert.not_valid_after - now
+            print(previous_ttl)
+
+        print("****************************************")
+        print("[SECURITY WARNING] Unrecognised certificate!")
+        print("The certificate presented for {} ({}) has never been seen before.".format(host, address))
+        print("This MIGHT be a Man-in-the-Middle attack.")
+        print("A different certificate has previously been seen {} times.".format(max_count))
+        if _HAS_CRYPTOGRAPHY:
+            if previous_ttl < datetime.timedelta():
+                print("That certificate has expired, which reduces suspicion somewhat.")
+            else:
+                print("That certificate is still valid for: {}".format(previous_ttl))
+        print("****************************************")
+        print("Attempt to verify the new certificate fingerprint out-of-band:")
+        print(fingerprint)
+        if automatic_choice:
+            choice = automatic_choice
         else:
-            certdir = os.path.join(xdg("config"), "cert_cache")
-            with open(os.path.join(certdir, most_frequent_cert+".crt"), "rb") as fp:
-                previous_cert = fp.read()
-            if _HAS_CRYPTOGRAPHY:
-                # Load the most frequently seen certificate to see if it has
-                # expired
-                previous_cert = x509.load_der_x509_certificate(previous_cert, _BACKEND)
-                previous_ttl = previous_cert.not_valid_after - now
-                print(previous_ttl)
+            choice = input("Accept this new certificate? Y/N ").strip().lower()
+        if choice in ("y", "yes"):
+            with open(os.path.join(sitedir, fingerprint), "w") as fp:
+                fp.write("1")
+            with open(os.path.join(certcache, fingerprint+".crt"), "wb") as fp:
+                fp.write(cert)
+        else:
+            raise Exception("TOFU Failure!")
 
-            print("****************************************")
-            print("[SECURITY WARNING] Unrecognised certificate!")
-            print("The certificate presented for {} ({}) has never been seen before.".format(host, address))
-            print("This MIGHT be a Man-in-the-Middle attack.")
-            print("A different certificate has previously been seen {} times.".format(max_count))
-            if _HAS_CRYPTOGRAPHY:
-                if previous_ttl < datetime.timedelta():
-                    print("That certificate has expired, which reduces suspicion somewhat.")
-                else:
-                    print("That certificate is still valid for: {}".format(previous_ttl))
-            print("****************************************")
-            print("Attempt to verify the new certificate fingerprint out-of-band:")
-            print(fingerprint)
-            if automatic_choice:
-                choice = automatic_choice
-            else:
-                choice = input("Accept this new certificate? Y/N ").strip().lower()
-            if choice in ("y", "yes"):
-                db_cur.execute("""INSERT INTO cert_cache
-                                VALUES (?, ?, ?, ?, ?, ?)""",
-                            (host, address, fingerprint, now, now, 1))
-                db_conn.commit()
-                with open(os.path.join(certdir, fingerprint+".crt"), "wb") as fp:
-                    fp.write(cert)
-            else:
-                raise Exception("TOFU Failure!")
-
-    # If not, cache this cert
-    else:
-        db_cur.execute("""INSERT INTO cert_cache
-            VALUES (?, ?, ?, ?, ?, ?)""",
-            (host, address, fingerprint, now, now, 1))
-        db_conn.commit()
-        certdir = os.path.join(xdg("config"), "cert_cache")
-        if not os.path.exists(certdir):
+    #3. If no directory or no cert found in it, we cache it
+    if not most_frequent_cert:
+        if not os.path.exists(certdir): # XDG_DATA/offpunk/certs
             os.makedirs(certdir)
-        with open(os.path.join(certdir, fingerprint+".crt"), "wb") as fp:
+        if not os.path.exists(hostdir): # XDG_DATA/offpunk/certs/site.net
+            os.makedirs(hostdir)
+        if not os.path.exists(sitedir): # XDG_DATA/offpunk/certs/site.net/123.123.123.123
+            os.makedirs(sitedir)
+
+        with open(os.path.join(sitedir, fingerprint), "w") as fp:
+            fp.write("1")
+        certcache = os.path.join(xdg("config"), "cert_cache")
+        if not os.path.exists(certcache):
+            os.makedirs(certcache)
+        with open(os.path.join(certcache, fingerprint+".crt"), "wb") as fp:
             fp.write(cert)
 
 def _fetch_gemini(url,timeout=DEFAULT_TIMEOUT,interactive=True,accept_bad_ssl_certificates=False,\
@@ -656,6 +666,9 @@ def _fetch_gemini(url,timeout=DEFAULT_TIMEOUT,interactive=True,accept_bad_ssl_ce
     # Send request and wrap response in a file descriptor
     url = urllib.parse.urlparse(url)
     new_netloc = host
+    #Handle IPV6 hostname
+    if ":" in new_netloc:
+        new_netloc = "[" + new_netloc + "]"
     if port != standard_ports["gemini"]:
         new_netloc += ":" + str(port)
     url = urllib.parse.urlunparse(url._replace(netloc=new_netloc))
@@ -799,8 +812,10 @@ def fetch(url,offline=False,download_image_first=True,images_mode="readable",val
                 path=_fetch_finger(url,**kwargs)
             elif scheme == "gemini":
                 path,newurl=_fetch_gemini(url,**kwargs)
+            elif scheme == "spartan":
+                path,newurl=_fetch_spartan(url,**kwargs)
             else:
-                print("scheme %s not implemented yet")
+                print("scheme %s not implemented yet"%scheme)
         except UserAbortException:
             return None, newurl
         except Exception as err:
